
#include "common.h"

#include <errno.h>

SEXP ps__define_errno(void) {

  SEXP env = PROTECT(Rf_allocSExp(ENVSXP));

#define PS_ADD_ERRNO(err,str,val)                                        \
  defineVar(                                                             \
    install(#err),                                                       \
    PROTECT(list2(PROTECT(ScalarInteger(val)), PROTECT(mkString(str)))), \
    env);	                                                         \
  UNPROTECT(3)


#ifdef EPERM
  PS_ADD_ERRNO(EPERM,"Operation not permitted.",EPERM);
#else
  PS_ADD_ERRNO(EPERM,"Operation not permitted.",NA_INTEGER);
#endif


#ifdef ENOENT
  PS_ADD_ERRNO(ENOENT,"No such file or directory.",ENOENT);
#else
  PS_ADD_ERRNO(ENOENT,"No such file or directory.",NA_INTEGER);
#endif


#ifdef ESRCH
  PS_ADD_ERRNO(ESRCH,"No such process.",ESRCH);
#else
  PS_ADD_ERRNO(ESRCH,"No such process.",NA_INTEGER);
#endif


#ifdef EINTR
  PS_ADD_ERRNO(EINTR,"Interrupted function call.",EINTR);
#else
  PS_ADD_ERRNO(EINTR,"Interrupted function call.",NA_INTEGER);
#endif


#ifdef EIO
  PS_ADD_ERRNO(EIO,"Input/output error.",EIO);
#else
  PS_ADD_ERRNO(EIO,"Input/output error.",NA_INTEGER);
#endif


#ifdef ENXIO
  PS_ADD_ERRNO(ENXIO,"No such device or address.",ENXIO);
#else
  PS_ADD_ERRNO(ENXIO,"No such device or address.",NA_INTEGER);
#endif


#ifdef E2BIG
  PS_ADD_ERRNO(E2BIG,"Arg list too long.",E2BIG);
#else
  PS_ADD_ERRNO(E2BIG,"Arg list too long.",NA_INTEGER);
#endif


#ifdef ENOEXEC
  PS_ADD_ERRNO(ENOEXEC,"Exec format error.",ENOEXEC);
#else
  PS_ADD_ERRNO(ENOEXEC,"Exec format error.",NA_INTEGER);
#endif


#ifdef EBADF
  PS_ADD_ERRNO(EBADF,"Bad file descriptor.",EBADF);
#else
  PS_ADD_ERRNO(EBADF,"Bad file descriptor.",NA_INTEGER);
#endif


#ifdef ECHILD
  PS_ADD_ERRNO(ECHILD,"No child processes.",ECHILD);
#else
  PS_ADD_ERRNO(ECHILD,"No child processes.",NA_INTEGER);
#endif


#ifdef EDEADLK
  PS_ADD_ERRNO(EDEADLK,"Resource deadlock avoided.",EDEADLK);
#else
  PS_ADD_ERRNO(EDEADLK,"Resource deadlock avoided.",NA_INTEGER);
#endif


#ifdef ENOMEM
  PS_ADD_ERRNO(ENOMEM,"Cannot allocate memory.",ENOMEM);
#else
  PS_ADD_ERRNO(ENOMEM,"Cannot allocate memory.",NA_INTEGER);
#endif


#ifdef EACCES
  PS_ADD_ERRNO(EACCES,"Permission denied.",EACCES);
#else
  PS_ADD_ERRNO(EACCES,"Permission denied.",NA_INTEGER);
#endif


#ifdef EFAULT
  PS_ADD_ERRNO(EFAULT,"Bad address.",EFAULT);
#else
  PS_ADD_ERRNO(EFAULT,"Bad address.",NA_INTEGER);
#endif


#ifdef ENOTBLK
  PS_ADD_ERRNO(ENOTBLK,"Not a block device.",ENOTBLK);
#else
  PS_ADD_ERRNO(ENOTBLK,"Not a block device.",NA_INTEGER);
#endif


#ifdef EBUSY
  PS_ADD_ERRNO(EBUSY,"Resource busy.",EBUSY);
#else
  PS_ADD_ERRNO(EBUSY,"Resource busy.",NA_INTEGER);
#endif


#ifdef EEXIST
  PS_ADD_ERRNO(EEXIST,"File exists.",EEXIST);
#else
  PS_ADD_ERRNO(EEXIST,"File exists.",NA_INTEGER);
#endif


#ifdef EXDEV
  PS_ADD_ERRNO(EXDEV,"Improper link.",EXDEV);
#else
  PS_ADD_ERRNO(EXDEV,"Improper link.",NA_INTEGER);
#endif


#ifdef ENODEV
  PS_ADD_ERRNO(ENODEV,"Operation not supported by device.",ENODEV);
#else
  PS_ADD_ERRNO(ENODEV,"Operation not supported by device.",NA_INTEGER);
#endif


#ifdef ENOTDIR
  PS_ADD_ERRNO(ENOTDIR,"Not a directory.",ENOTDIR);
#else
  PS_ADD_ERRNO(ENOTDIR,"Not a directory.",NA_INTEGER);
#endif


#ifdef EISDIR
  PS_ADD_ERRNO(EISDIR,"Is a directory.",EISDIR);
#else
  PS_ADD_ERRNO(EISDIR,"Is a directory.",NA_INTEGER);
#endif


#ifdef EINVAL
  PS_ADD_ERRNO(EINVAL,"Invalid argument.",EINVAL);
#else
  PS_ADD_ERRNO(EINVAL,"Invalid argument.",NA_INTEGER);
#endif


#ifdef ENFILE
  PS_ADD_ERRNO(ENFILE,"Too many open files in system.",ENFILE);
#else
  PS_ADD_ERRNO(ENFILE,"Too many open files in system.",NA_INTEGER);
#endif


#ifdef EMFILE
  PS_ADD_ERRNO(EMFILE,"Too many open files.",EMFILE);
#else
  PS_ADD_ERRNO(EMFILE,"Too many open files.",NA_INTEGER);
#endif


#ifdef ENOTTY
  PS_ADD_ERRNO(ENOTTY,"Inappropriate ioctl for device.",ENOTTY);
#else
  PS_ADD_ERRNO(ENOTTY,"Inappropriate ioctl for device.",NA_INTEGER);
#endif


#ifdef ETXTBSY
  PS_ADD_ERRNO(ETXTBSY,"Text file busy.",ETXTBSY);
#else
  PS_ADD_ERRNO(ETXTBSY,"Text file busy.",NA_INTEGER);
#endif


#ifdef EFBIG
  PS_ADD_ERRNO(EFBIG,"File too large.",EFBIG);
#else
  PS_ADD_ERRNO(EFBIG,"File too large.",NA_INTEGER);
#endif


#ifdef ENOSPC
  PS_ADD_ERRNO(ENOSPC,"Device out of space.",ENOSPC);
#else
  PS_ADD_ERRNO(ENOSPC,"Device out of space.",NA_INTEGER);
#endif


#ifdef ESPIPE
  PS_ADD_ERRNO(ESPIPE,"Illegal seek.",ESPIPE);
#else
  PS_ADD_ERRNO(ESPIPE,"Illegal seek.",NA_INTEGER);
#endif


#ifdef EROFS
  PS_ADD_ERRNO(EROFS,"Read-only file system.",EROFS);
#else
  PS_ADD_ERRNO(EROFS,"Read-only file system.",NA_INTEGER);
#endif


#ifdef EMLINK
  PS_ADD_ERRNO(EMLINK,"Too many links.",EMLINK);
#else
  PS_ADD_ERRNO(EMLINK,"Too many links.",NA_INTEGER);
#endif


#ifdef EPIPE
  PS_ADD_ERRNO(EPIPE,"Broken pipe.",EPIPE);
#else
  PS_ADD_ERRNO(EPIPE,"Broken pipe.",NA_INTEGER);
#endif


#ifdef EDOM
  PS_ADD_ERRNO(EDOM,"Numerical argument out of domain.",EDOM);
#else
  PS_ADD_ERRNO(EDOM,"Numerical argument out of domain.",NA_INTEGER);
#endif


#ifdef ERANGE
  PS_ADD_ERRNO(ERANGE,"Numerical result out of range.",ERANGE);
#else
  PS_ADD_ERRNO(ERANGE,"Numerical result out of range.",NA_INTEGER);
#endif


#ifdef EAGAIN
  PS_ADD_ERRNO(EAGAIN,"Resource temporarily unavailable.",EAGAIN);
#else
  PS_ADD_ERRNO(EAGAIN,"Resource temporarily unavailable.",NA_INTEGER);
#endif


#ifdef EINPROGRESS
  PS_ADD_ERRNO(EINPROGRESS,"Operation now in progress.",EINPROGRESS);
#else
  PS_ADD_ERRNO(EINPROGRESS,"Operation now in progress.",NA_INTEGER);
#endif


#ifdef EALREADY
  PS_ADD_ERRNO(EALREADY,"Operation already in progress.",EALREADY);
#else
  PS_ADD_ERRNO(EALREADY,"Operation already in progress.",NA_INTEGER);
#endif


#ifdef ENOTSOCK
  PS_ADD_ERRNO(ENOTSOCK,"Socket operation on non-socket.",ENOTSOCK);
#else
  PS_ADD_ERRNO(ENOTSOCK,"Socket operation on non-socket.",NA_INTEGER);
#endif


#ifdef EDESTADDRREQ
  PS_ADD_ERRNO(EDESTADDRREQ,"Destination address required.",EDESTADDRREQ);
#else
  PS_ADD_ERRNO(EDESTADDRREQ,"Destination address required.",NA_INTEGER);
#endif


#ifdef EMSGSIZE
  PS_ADD_ERRNO(EMSGSIZE,"Message too long.",EMSGSIZE);
#else
  PS_ADD_ERRNO(EMSGSIZE,"Message too long.",NA_INTEGER);
#endif


#ifdef EPROTOTYPE
  PS_ADD_ERRNO(EPROTOTYPE,"Protocol wrong type for socket.",EPROTOTYPE);
#else
  PS_ADD_ERRNO(EPROTOTYPE,"Protocol wrong type for socket.",NA_INTEGER);
#endif


#ifdef ENOPROTOOPT
  PS_ADD_ERRNO(ENOPROTOOPT,"Protocol not available.",ENOPROTOOPT);
#else
  PS_ADD_ERRNO(ENOPROTOOPT,"Protocol not available.",NA_INTEGER);
#endif


#ifdef EPROTONOSUPPORT
  PS_ADD_ERRNO(EPROTONOSUPPORT,"Protocol not supported.",EPROTONOSUPPORT);
#else
  PS_ADD_ERRNO(EPROTONOSUPPORT,"Protocol not supported.",NA_INTEGER);
#endif


#ifdef ESOCKTNOSUPPORT
  PS_ADD_ERRNO(ESOCKTNOSUPPORT,"Socket type not supported.",ESOCKTNOSUPPORT);
#else
  PS_ADD_ERRNO(ESOCKTNOSUPPORT,"Socket type not supported.",NA_INTEGER);
#endif


#ifdef ENOTSUP
  PS_ADD_ERRNO(ENOTSUP,"Not supported.",ENOTSUP);
#else
  PS_ADD_ERRNO(ENOTSUP,"Not supported.",NA_INTEGER);
#endif


#ifdef EPFNOSUPPORT
  PS_ADD_ERRNO(EPFNOSUPPORT,"Protocol family not supported.",EPFNOSUPPORT);
#else
  PS_ADD_ERRNO(EPFNOSUPPORT,"Protocol family not supported.",NA_INTEGER);
#endif


#ifdef EAFNOSUPPORT
  PS_ADD_ERRNO(EAFNOSUPPORT,"Address family not supported by protocol family.",EAFNOSUPPORT);
#else
  PS_ADD_ERRNO(EAFNOSUPPORT,"Address family not supported by protocol family.",NA_INTEGER);
#endif


#ifdef EADDRINUSE
  PS_ADD_ERRNO(EADDRINUSE,"Address already in use.",EADDRINUSE);
#else
  PS_ADD_ERRNO(EADDRINUSE,"Address already in use.",NA_INTEGER);
#endif


#ifdef EADDRNOTAVAIL
  PS_ADD_ERRNO(EADDRNOTAVAIL,"Cannot assign requested address.",EADDRNOTAVAIL);
#else
  PS_ADD_ERRNO(EADDRNOTAVAIL,"Cannot assign requested address.",NA_INTEGER);
#endif


#ifdef ENETDOWN
  PS_ADD_ERRNO(ENETDOWN,"Network is down.",ENETDOWN);
#else
  PS_ADD_ERRNO(ENETDOWN,"Network is down.",NA_INTEGER);
#endif


#ifdef ENETUNREACH
  PS_ADD_ERRNO(ENETUNREACH,"Network is unreachable.",ENETUNREACH);
#else
  PS_ADD_ERRNO(ENETUNREACH,"Network is unreachable.",NA_INTEGER);
#endif


#ifdef ENETRESET
  PS_ADD_ERRNO(ENETRESET,"Network dropped connection on reset.",ENETRESET);
#else
  PS_ADD_ERRNO(ENETRESET,"Network dropped connection on reset.",NA_INTEGER);
#endif


#ifdef ECONNABORTED
  PS_ADD_ERRNO(ECONNABORTED,"Software caused connection abort.",ECONNABORTED);
#else
  PS_ADD_ERRNO(ECONNABORTED,"Software caused connection abort.",NA_INTEGER);
#endif


#ifdef ECONNRESET
  PS_ADD_ERRNO(ECONNRESET,"Connection reset by peer.",ECONNRESET);
#else
  PS_ADD_ERRNO(ECONNRESET,"Connection reset by peer.",NA_INTEGER);
#endif


#ifdef ENOBUFS
  PS_ADD_ERRNO(ENOBUFS,"No buffer space available.",ENOBUFS);
#else
  PS_ADD_ERRNO(ENOBUFS,"No buffer space available.",NA_INTEGER);
#endif


#ifdef EISCONN
  PS_ADD_ERRNO(EISCONN,"Socket is already connected.",EISCONN);
#else
  PS_ADD_ERRNO(EISCONN,"Socket is already connected.",NA_INTEGER);
#endif


#ifdef ENOTCONN
  PS_ADD_ERRNO(ENOTCONN,"Socket is not connected.",ENOTCONN);
#else
  PS_ADD_ERRNO(ENOTCONN,"Socket is not connected.",NA_INTEGER);
#endif


#ifdef ESHUTDOWN
  PS_ADD_ERRNO(ESHUTDOWN,"Cannot send after socket shutdown.",ESHUTDOWN);
#else
  PS_ADD_ERRNO(ESHUTDOWN,"Cannot send after socket shutdown.",NA_INTEGER);
#endif


#ifdef ETIMEDOUT
  PS_ADD_ERRNO(ETIMEDOUT,"Operation timed out.",ETIMEDOUT);
#else
  PS_ADD_ERRNO(ETIMEDOUT,"Operation timed out.",NA_INTEGER);
#endif


#ifdef ECONNREFUSED
  PS_ADD_ERRNO(ECONNREFUSED,"Connection refused.",ECONNREFUSED);
#else
  PS_ADD_ERRNO(ECONNREFUSED,"Connection refused.",NA_INTEGER);
#endif


#ifdef ELOOP
  PS_ADD_ERRNO(ELOOP,"Too many levels of symbolic links.",ELOOP);
#else
  PS_ADD_ERRNO(ELOOP,"Too many levels of symbolic links.",NA_INTEGER);
#endif


#ifdef ENAMETOOLONG
  PS_ADD_ERRNO(ENAMETOOLONG,"File name too long.",ENAMETOOLONG);
#else
  PS_ADD_ERRNO(ENAMETOOLONG,"File name too long.",NA_INTEGER);
#endif


#ifdef EHOSTDOWN
  PS_ADD_ERRNO(EHOSTDOWN,"Host is down.",EHOSTDOWN);
#else
  PS_ADD_ERRNO(EHOSTDOWN,"Host is down.",NA_INTEGER);
#endif


#ifdef EHOSTUNREACH
  PS_ADD_ERRNO(EHOSTUNREACH,"No route to host.",EHOSTUNREACH);
#else
  PS_ADD_ERRNO(EHOSTUNREACH,"No route to host.",NA_INTEGER);
#endif


#ifdef ENOTEMPTY
  PS_ADD_ERRNO(ENOTEMPTY,"Directory not empty.",ENOTEMPTY);
#else
  PS_ADD_ERRNO(ENOTEMPTY,"Directory not empty.",NA_INTEGER);
#endif


#ifdef EPROCLIM
  PS_ADD_ERRNO(EPROCLIM,"Too many processes.",EPROCLIM);
#else
  PS_ADD_ERRNO(EPROCLIM,"Too many processes.",NA_INTEGER);
#endif


#ifdef EUSERS
  PS_ADD_ERRNO(EUSERS,"Too many users.",EUSERS);
#else
  PS_ADD_ERRNO(EUSERS,"Too many users.",NA_INTEGER);
#endif


#ifdef EDQUOT
  PS_ADD_ERRNO(EDQUOT,"Disc quota exceeded.",EDQUOT);
#else
  PS_ADD_ERRNO(EDQUOT,"Disc quota exceeded.",NA_INTEGER);
#endif


#ifdef ESTALE
  PS_ADD_ERRNO(ESTALE,"Stale NFS file handle.",ESTALE);
#else
  PS_ADD_ERRNO(ESTALE,"Stale NFS file handle.",NA_INTEGER);
#endif


#ifdef EBADRPC
  PS_ADD_ERRNO(EBADRPC,"RPC struct is bad.",EBADRPC);
#else
  PS_ADD_ERRNO(EBADRPC,"RPC struct is bad.",NA_INTEGER);
#endif


#ifdef ERPCMISMATCH
  PS_ADD_ERRNO(ERPCMISMATCH,"RPC version wrong.",ERPCMISMATCH);
#else
  PS_ADD_ERRNO(ERPCMISMATCH,"RPC version wrong.",NA_INTEGER);
#endif


#ifdef EPROGUNAVAIL
  PS_ADD_ERRNO(EPROGUNAVAIL,"RPC prog. not avail.",EPROGUNAVAIL);
#else
  PS_ADD_ERRNO(EPROGUNAVAIL,"RPC prog. not avail.",NA_INTEGER);
#endif


#ifdef EPROGMISMATCH
  PS_ADD_ERRNO(EPROGMISMATCH,"Program version wrong.",EPROGMISMATCH);
#else
  PS_ADD_ERRNO(EPROGMISMATCH,"Program version wrong.",NA_INTEGER);
#endif


#ifdef EPROCUNAVAIL
  PS_ADD_ERRNO(EPROCUNAVAIL,"Bad procedure for program.",EPROCUNAVAIL);
#else
  PS_ADD_ERRNO(EPROCUNAVAIL,"Bad procedure for program.",NA_INTEGER);
#endif


#ifdef ENOLCK
  PS_ADD_ERRNO(ENOLCK,"No locks available.",ENOLCK);
#else
  PS_ADD_ERRNO(ENOLCK,"No locks available.",NA_INTEGER);
#endif


#ifdef ENOSYS
  PS_ADD_ERRNO(ENOSYS,"Function not implemented.",ENOSYS);
#else
  PS_ADD_ERRNO(ENOSYS,"Function not implemented.",NA_INTEGER);
#endif


#ifdef EFTYPE
  PS_ADD_ERRNO(EFTYPE,"Inappropriate file type or format.",EFTYPE);
#else
  PS_ADD_ERRNO(EFTYPE,"Inappropriate file type or format.",NA_INTEGER);
#endif


#ifdef EAUTH
  PS_ADD_ERRNO(EAUTH,"Authentication error.",EAUTH);
#else
  PS_ADD_ERRNO(EAUTH,"Authentication error.",NA_INTEGER);
#endif


#ifdef ENEEDAUTH
  PS_ADD_ERRNO(ENEEDAUTH,"Need authenticator.",ENEEDAUTH);
#else
  PS_ADD_ERRNO(ENEEDAUTH,"Need authenticator.",NA_INTEGER);
#endif


#ifdef EPWROFF
  PS_ADD_ERRNO(EPWROFF,"Device power is off.",EPWROFF);
#else
  PS_ADD_ERRNO(EPWROFF,"Device power is off.",NA_INTEGER);
#endif


#ifdef EDEVERR
  PS_ADD_ERRNO(EDEVERR,"Device error.",EDEVERR);
#else
  PS_ADD_ERRNO(EDEVERR,"Device error.",NA_INTEGER);
#endif


#ifdef EOVERFLOW
  PS_ADD_ERRNO(EOVERFLOW,"Value too large to be stored in data type.",EOVERFLOW);
#else
  PS_ADD_ERRNO(EOVERFLOW,"Value too large to be stored in data type.",NA_INTEGER);
#endif


#ifdef EBADEXEC
  PS_ADD_ERRNO(EBADEXEC,"Bad executable (or shared library).",EBADEXEC);
#else
  PS_ADD_ERRNO(EBADEXEC,"Bad executable (or shared library).",NA_INTEGER);
#endif


#ifdef EBADARCH
  PS_ADD_ERRNO(EBADARCH,"Bad CPU type in executable.",EBADARCH);
#else
  PS_ADD_ERRNO(EBADARCH,"Bad CPU type in executable.",NA_INTEGER);
#endif


#ifdef ESHLIBVERS
  PS_ADD_ERRNO(ESHLIBVERS,"Shared library version mismatch.",ESHLIBVERS);
#else
  PS_ADD_ERRNO(ESHLIBVERS,"Shared library version mismatch.",NA_INTEGER);
#endif


#ifdef EBADMACHO
  PS_ADD_ERRNO(EBADMACHO,"Malformed Mach-o file.",EBADMACHO);
#else
  PS_ADD_ERRNO(EBADMACHO,"Malformed Mach-o file.",NA_INTEGER);
#endif


#ifdef ECANCELED
  PS_ADD_ERRNO(ECANCELED,"Operation canceled.",ECANCELED);
#else
  PS_ADD_ERRNO(ECANCELED,"Operation canceled.",NA_INTEGER);
#endif


#ifdef EIDRM
  PS_ADD_ERRNO(EIDRM,"Identifier removed.",EIDRM);
#else
  PS_ADD_ERRNO(EIDRM,"Identifier removed.",NA_INTEGER);
#endif


#ifdef ENOMSG
  PS_ADD_ERRNO(ENOMSG,"No message of desired type.",ENOMSG);
#else
  PS_ADD_ERRNO(ENOMSG,"No message of desired type.",NA_INTEGER);
#endif


#ifdef EILSEQ
  PS_ADD_ERRNO(EILSEQ,"Illegal byte sequence.",EILSEQ);
#else
  PS_ADD_ERRNO(EILSEQ,"Illegal byte sequence.",NA_INTEGER);
#endif


#ifdef ENOATTR
  PS_ADD_ERRNO(ENOATTR,"Attribute not found.",ENOATTR);
#else
  PS_ADD_ERRNO(ENOATTR,"Attribute not found.",NA_INTEGER);
#endif


#ifdef EBADMSG
  PS_ADD_ERRNO(EBADMSG,"Bad message.",EBADMSG);
#else
  PS_ADD_ERRNO(EBADMSG,"Bad message.",NA_INTEGER);
#endif


#ifdef EMULTIHOP
  PS_ADD_ERRNO(EMULTIHOP,"Multihop attempted.",EMULTIHOP);
#else
  PS_ADD_ERRNO(EMULTIHOP,"Multihop attempted.",NA_INTEGER);
#endif


#ifdef ENODATA
  PS_ADD_ERRNO(ENODATA,"No message available.",ENODATA);
#else
  PS_ADD_ERRNO(ENODATA,"No message available.",NA_INTEGER);
#endif


#ifdef ENOSTR
  PS_ADD_ERRNO(ENOSTR,"Not a STREAM.",ENOSTR);
#else
  PS_ADD_ERRNO(ENOSTR,"Not a STREAM.",NA_INTEGER);
#endif


#ifdef EPROTO
  PS_ADD_ERRNO(EPROTO,"Protocol error.",EPROTO);
#else
  PS_ADD_ERRNO(EPROTO,"Protocol error.",NA_INTEGER);
#endif


#ifdef ETIME
  PS_ADD_ERRNO(ETIME,"STREAM ioctl() timeout.",ETIME);
#else
  PS_ADD_ERRNO(ETIME,"STREAM ioctl() timeout.",NA_INTEGER);
#endif


#ifdef EOPNOTSUPP
  PS_ADD_ERRNO(EOPNOTSUPP,"Operation not supported on socket.",EOPNOTSUPP);
#else
  PS_ADD_ERRNO(EOPNOTSUPP,"Operation not supported on socket.",NA_INTEGER);
#endif


#ifdef EWOULDBLOCK
  PS_ADD_ERRNO(EWOULDBLOCK,"Resource temporarily unavailable.",EWOULDBLOCK);
#else
  PS_ADD_ERRNO(EWOULDBLOCK,"Resource temporarily unavailable.",NA_INTEGER);
#endif


#ifdef ETOOMANYREFS
  PS_ADD_ERRNO(ETOOMANYREFS,"Too many references: cannot splice.",ETOOMANYREFS);
#else
  PS_ADD_ERRNO(ETOOMANYREFS,"Too many references: cannot splice.",NA_INTEGER);
#endif


#ifdef EREMOTE
  PS_ADD_ERRNO(EREMOTE,"File is already NFS-mounted",EREMOTE);
#else
  PS_ADD_ERRNO(EREMOTE,"File is already NFS-mounted",NA_INTEGER);
#endif


#ifdef EBACKGROUND
  PS_ADD_ERRNO(EBACKGROUND,"Caller not in the foreground process group",EBACKGROUND);
#else
  PS_ADD_ERRNO(EBACKGROUND,"Caller not in the foreground process group",NA_INTEGER);
#endif


#ifdef EDIED
  PS_ADD_ERRNO(EDIED,"Translator died",EDIED);
#else
  PS_ADD_ERRNO(EDIED,"Translator died",NA_INTEGER);
#endif


#ifdef ED
  PS_ADD_ERRNO(ED,"The experienced user will know what is wrong.#else",ED);
#else
  PS_ADD_ERRNO(ED,"The experienced user will know what is wrong.#else",NA_INTEGER);
#endif


#ifdef EGREGIOUS
  PS_ADD_ERRNO(EGREGIOUS,"You did *what*?",EGREGIOUS);
#else
  PS_ADD_ERRNO(EGREGIOUS,"You did *what*?",NA_INTEGER);
#endif


#ifdef EIEIO
  PS_ADD_ERRNO(EIEIO,"Go home and have a glass of warm, dairy-fresh milk.",EIEIO);
#else
  PS_ADD_ERRNO(EIEIO,"Go home and have a glass of warm, dairy-fresh milk.",NA_INTEGER);
#endif


#ifdef EGRATUITOUS
  PS_ADD_ERRNO(EGRATUITOUS,"This error code has no purpose.",EGRATUITOUS);
#else
  PS_ADD_ERRNO(EGRATUITOUS,"This error code has no purpose.",NA_INTEGER);
#endif


#ifdef ENOLINK
  PS_ADD_ERRNO(ENOLINK,"Link has been severed.",ENOLINK);
#else
  PS_ADD_ERRNO(ENOLINK,"Link has been severed.",NA_INTEGER);
#endif


#ifdef ENOSR
  PS_ADD_ERRNO(ENOSR,"Out of streams resources.",ENOSR);
#else
  PS_ADD_ERRNO(ENOSR,"Out of streams resources.",NA_INTEGER);
#endif


#ifdef ERESTART
  PS_ADD_ERRNO(ERESTART,"Interrupted system call should be restarted.",ERESTART);
#else
  PS_ADD_ERRNO(ERESTART,"Interrupted system call should be restarted.",NA_INTEGER);
#endif


#ifdef ECHRNG
  PS_ADD_ERRNO(ECHRNG,"Channel number out of range.",ECHRNG);
#else
  PS_ADD_ERRNO(ECHRNG,"Channel number out of range.",NA_INTEGER);
#endif


#ifdef EL2NSYNC
  PS_ADD_ERRNO(EL2NSYNC,"Level 2 not synchronized.",EL2NSYNC);
#else
  PS_ADD_ERRNO(EL2NSYNC,"Level 2 not synchronized.",NA_INTEGER);
#endif


#ifdef EL3HLT
  PS_ADD_ERRNO(EL3HLT,"Level 3 halted.",EL3HLT);
#else
  PS_ADD_ERRNO(EL3HLT,"Level 3 halted.",NA_INTEGER);
#endif


#ifdef EL3RST
  PS_ADD_ERRNO(EL3RST,"Level 3 reset.",EL3RST);
#else
  PS_ADD_ERRNO(EL3RST,"Level 3 reset.",NA_INTEGER);
#endif


#ifdef ELNRNG
  PS_ADD_ERRNO(ELNRNG,"Link number out of range.",ELNRNG);
#else
  PS_ADD_ERRNO(ELNRNG,"Link number out of range.",NA_INTEGER);
#endif


#ifdef EUNATCH
  PS_ADD_ERRNO(EUNATCH,"Protocol driver not attached.",EUNATCH);
#else
  PS_ADD_ERRNO(EUNATCH,"Protocol driver not attached.",NA_INTEGER);
#endif


#ifdef ENOCSI
  PS_ADD_ERRNO(ENOCSI,"No CSI structure available.",ENOCSI);
#else
  PS_ADD_ERRNO(ENOCSI,"No CSI structure available.",NA_INTEGER);
#endif


#ifdef EL2HLT
  PS_ADD_ERRNO(EL2HLT,"Level 2 halted.",EL2HLT);
#else
  PS_ADD_ERRNO(EL2HLT,"Level 2 halted.",NA_INTEGER);
#endif


#ifdef EBADE
  PS_ADD_ERRNO(EBADE,"Invalid exchange.",EBADE);
#else
  PS_ADD_ERRNO(EBADE,"Invalid exchange.",NA_INTEGER);
#endif


#ifdef EBADR
  PS_ADD_ERRNO(EBADR,"Invalid request descriptor.",EBADR);
#else
  PS_ADD_ERRNO(EBADR,"Invalid request descriptor.",NA_INTEGER);
#endif


#ifdef EXFULL
  PS_ADD_ERRNO(EXFULL,"Exchange full.",EXFULL);
#else
  PS_ADD_ERRNO(EXFULL,"Exchange full.",NA_INTEGER);
#endif


#ifdef ENOANO
  PS_ADD_ERRNO(ENOANO,"No anode.",ENOANO);
#else
  PS_ADD_ERRNO(ENOANO,"No anode.",NA_INTEGER);
#endif


#ifdef EBADRQC
  PS_ADD_ERRNO(EBADRQC,"Invalid request code.",EBADRQC);
#else
  PS_ADD_ERRNO(EBADRQC,"Invalid request code.",NA_INTEGER);
#endif


#ifdef EBADSLT
  PS_ADD_ERRNO(EBADSLT,"Invalid slot.",EBADSLT);
#else
  PS_ADD_ERRNO(EBADSLT,"Invalid slot.",NA_INTEGER);
#endif


#ifdef EDEADLOCK
  PS_ADD_ERRNO(EDEADLOCK,"File locking deadlock error.",EDEADLOCK);
#else
  PS_ADD_ERRNO(EDEADLOCK,"File locking deadlock error.",NA_INTEGER);
#endif


#ifdef EBFONT
  PS_ADD_ERRNO(EBFONT,"Bad font file format.",EBFONT);
#else
  PS_ADD_ERRNO(EBFONT,"Bad font file format.",NA_INTEGER);
#endif


#ifdef ENONET
  PS_ADD_ERRNO(ENONET,"Machine is not on the network.",ENONET);
#else
  PS_ADD_ERRNO(ENONET,"Machine is not on the network.",NA_INTEGER);
#endif


#ifdef ENOPKG
  PS_ADD_ERRNO(ENOPKG,"Package not installed.",ENOPKG);
#else
  PS_ADD_ERRNO(ENOPKG,"Package not installed.",NA_INTEGER);
#endif


#ifdef EADV
  PS_ADD_ERRNO(EADV,"Advertise error.",EADV);
#else
  PS_ADD_ERRNO(EADV,"Advertise error.",NA_INTEGER);
#endif


#ifdef ESRMNT
  PS_ADD_ERRNO(ESRMNT,"Srmount error.",ESRMNT);
#else
  PS_ADD_ERRNO(ESRMNT,"Srmount error.",NA_INTEGER);
#endif


#ifdef ECOMM
  PS_ADD_ERRNO(ECOMM,"Communication error on send.",ECOMM);
#else
  PS_ADD_ERRNO(ECOMM,"Communication error on send.",NA_INTEGER);
#endif


#ifdef EDOTDOT
  PS_ADD_ERRNO(EDOTDOT,"RFS specific error",EDOTDOT);
#else
  PS_ADD_ERRNO(EDOTDOT,"RFS specific error",NA_INTEGER);
#endif


#ifdef ENOTUNIQ
  PS_ADD_ERRNO(ENOTUNIQ,"Name not unique on network.",ENOTUNIQ);
#else
  PS_ADD_ERRNO(ENOTUNIQ,"Name not unique on network.",NA_INTEGER);
#endif


#ifdef EBADFD
  PS_ADD_ERRNO(EBADFD,"File descriptor in bad state.",EBADFD);
#else
  PS_ADD_ERRNO(EBADFD,"File descriptor in bad state.",NA_INTEGER);
#endif


#ifdef EREMCHG
  PS_ADD_ERRNO(EREMCHG,"Remote address changed.",EREMCHG);
#else
  PS_ADD_ERRNO(EREMCHG,"Remote address changed.",NA_INTEGER);
#endif


#ifdef ELIBACC
  PS_ADD_ERRNO(ELIBACC,"Can not access a needed shared library.",ELIBACC);
#else
  PS_ADD_ERRNO(ELIBACC,"Can not access a needed shared library.",NA_INTEGER);
#endif


#ifdef ELIBBAD
  PS_ADD_ERRNO(ELIBBAD,"Accessing a corrupted shared library.",ELIBBAD);
#else
  PS_ADD_ERRNO(ELIBBAD,"Accessing a corrupted shared library.",NA_INTEGER);
#endif


#ifdef ELIBSCN
  PS_ADD_ERRNO(ELIBSCN,".lib section in a.out corrupted.",ELIBSCN);
#else
  PS_ADD_ERRNO(ELIBSCN,".lib section in a.out corrupted.",NA_INTEGER);
#endif


#ifdef ELIBMAX
  PS_ADD_ERRNO(ELIBMAX,"Attempting to link in too many shared libraries.",ELIBMAX);
#else
  PS_ADD_ERRNO(ELIBMAX,"Attempting to link in too many shared libraries.",NA_INTEGER);
#endif


#ifdef ELIBEXEC
  PS_ADD_ERRNO(ELIBEXEC,"Cannot exec a shared library directly.",ELIBEXEC);
#else
  PS_ADD_ERRNO(ELIBEXEC,"Cannot exec a shared library directly.",NA_INTEGER);
#endif


#ifdef ESTRPIPE
  PS_ADD_ERRNO(ESTRPIPE,"Streams pipe error.",ESTRPIPE);
#else
  PS_ADD_ERRNO(ESTRPIPE,"Streams pipe error.",NA_INTEGER);
#endif


#ifdef EUCLEAN
  PS_ADD_ERRNO(EUCLEAN,"Structure needs cleaning.",EUCLEAN);
#else
  PS_ADD_ERRNO(EUCLEAN,"Structure needs cleaning.",NA_INTEGER);
#endif


#ifdef ENOTNAM
  PS_ADD_ERRNO(ENOTNAM,"Not a XENIX named type file.",ENOTNAM);
#else
  PS_ADD_ERRNO(ENOTNAM,"Not a XENIX named type file.",NA_INTEGER);
#endif


#ifdef ENAVAIL
  PS_ADD_ERRNO(ENAVAIL,"No XENIX semaphores available.",ENAVAIL);
#else
  PS_ADD_ERRNO(ENAVAIL,"No XENIX semaphores available.",NA_INTEGER);
#endif


#ifdef EISNAM
  PS_ADD_ERRNO(EISNAM,"Is a named type file.",EISNAM);
#else
  PS_ADD_ERRNO(EISNAM,"Is a named type file.",NA_INTEGER);
#endif


#ifdef EREMOTEIO
  PS_ADD_ERRNO(EREMOTEIO,"Remote I/O error.",EREMOTEIO);
#else
  PS_ADD_ERRNO(EREMOTEIO,"Remote I/O error.",NA_INTEGER);
#endif


#ifdef ENOMEDIUM
  PS_ADD_ERRNO(ENOMEDIUM,"No medium found.",ENOMEDIUM);
#else
  PS_ADD_ERRNO(ENOMEDIUM,"No medium found.",NA_INTEGER);
#endif


#ifdef EMEDIUMTYPE
  PS_ADD_ERRNO(EMEDIUMTYPE,"Wrong medium type.",EMEDIUMTYPE);
#else
  PS_ADD_ERRNO(EMEDIUMTYPE,"Wrong medium type.",NA_INTEGER);
#endif


#ifdef ENOKEY
  PS_ADD_ERRNO(ENOKEY,"Required key not available.",ENOKEY);
#else
  PS_ADD_ERRNO(ENOKEY,"Required key not available.",NA_INTEGER);
#endif


#ifdef EKEYEXPIRED
  PS_ADD_ERRNO(EKEYEXPIRED,"Key has expired.",EKEYEXPIRED);
#else
  PS_ADD_ERRNO(EKEYEXPIRED,"Key has expired.",NA_INTEGER);
#endif


#ifdef EKEYREVOKED
  PS_ADD_ERRNO(EKEYREVOKED,"Key has been revoked.",EKEYREVOKED);
#else
  PS_ADD_ERRNO(EKEYREVOKED,"Key has been revoked.",NA_INTEGER);
#endif


#ifdef EKEYREJECTED
  PS_ADD_ERRNO(EKEYREJECTED,"Key was rejected by service.",EKEYREJECTED);
#else
  PS_ADD_ERRNO(EKEYREJECTED,"Key was rejected by service.",NA_INTEGER);
#endif


#ifdef EOWNERDEAD
  PS_ADD_ERRNO(EOWNERDEAD,"Owner died.",EOWNERDEAD);
#else
  PS_ADD_ERRNO(EOWNERDEAD,"Owner died.",NA_INTEGER);
#endif


#ifdef ENOTRECOVERABLE
  PS_ADD_ERRNO(ENOTRECOVERABLE,"State not recoverable.",ENOTRECOVERABLE);
#else
  PS_ADD_ERRNO(ENOTRECOVERABLE,"State not recoverable.",NA_INTEGER);
#endif


#ifdef ERFKILL
  PS_ADD_ERRNO(ERFKILL,"Operation not possible due to RF-kill.",ERFKILL);
#else
  PS_ADD_ERRNO(ERFKILL,"Operation not possible due to RF-kill.",NA_INTEGER);
#endif


#ifdef EHWPOISON
  PS_ADD_ERRNO(EHWPOISON,"Memory page has hardware error.",EHWPOISON);
#else
  PS_ADD_ERRNO(EHWPOISON,"Memory page has hardware error.",NA_INTEGER);
#endif


#undef PS_ADD_ERRNO
#undef PS_ADD_ERRNOX

  UNPROTECT(1);
  return env;
}
